// Uebung3 Betriebssysteme 2
// Matthias Jauernig, 2006
// Sender-Programm

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/msg.h>
#include <sys/stat.h>
#include <stropts.h>

#define PIPENAME "pipe"
#define INITSEND 0
#define SENDDATA 1
#define ENDSEND  2

struct mymsg{
	long pid;
	char mtext[100];
};

int main(void){
	int rc, flags, fd;
	struct stat buf;
	struct strbuf ctl, data;	// Puffer fr Steuer- und Datenbereich
	FILE* fp;			// Pointer auf Pipe (File)
	mode_t mode;			// Modus zum Pipe-ffnen
	struct mymsg *msg = (struct mymsg*)malloc(sizeof(struct mymsg)); // zu sendende Daten
	char* cbuf = (char*)malloc(1);   // Puffer fr Steuerinformationen

	// Pipe-Test
	mode = 0644;
	if((rc=stat(PIPENAME, &buf)) < 0){
		if(errno != ENOENT){
			perror("stat");
			return 1;
		}
		if((rc=mkfifo(PIPENAME, mode)) < 0){
			perror("mkfifo");
			return 2;
		}
	}
	else{
		if(!S_ISFIFO(buf.st_mode)){
			printf("File %s existiert, ist aber keine Pipe.\n", PIPENAME);
			return 3;
		}
	}
	
	// vorhandene Pipe zum Schreiben oeffnen
	if((fp = fopen(PIPENAME, "w")) < 0){
		perror("fopen");
		return 4;
	}
	fd = fileno(fp);		// File Deskriptor holen

	msg->pid = getpid(); 		// Prozess-ID als Nachrichten-Kennzeichen verwenden
	
	// Daten fuer putmsg() setzen
	data.len = sizeof(msg->pid);	// bei Initial-Nachricht nur PID senden
	data.buf = (char*)msg;		// Datenbereich
	flags = 0;			// beliebige Nachrichten
	cbuf[0] = INITSEND;		// Sende-Initialisierung
	ctl.len = 1;    		// 1 Zeichen im Steuerbereich
	ctl.buf = cbuf;			// Steuerbereich
	
	// Empfaenger-Prozess ueber neuen Sender informieren
	if((rc=putmsg(fd, &ctl, &data, flags)) < 0){
		perror("Initiales putmsg()");
		return 5;
	}
	printf("Sender an Pipe angemeldet...\n");
	
	// Zeilen von stdin lesen und an Message Queue senden bis EOF
	cbuf[0] = SENDDATA;
	printf("Text eingeben\n");
	printf(": ");
	while((fgets(msg->mtext, sizeof(msg->mtext), stdin)) != NULL){
		data.len = strlen(msg->mtext)+1 + sizeof(msg->pid); // zu sendende Nachrichtenbytes
		if((rc=putmsg(fd, &ctl, &data, flags)) < 0){
			perror("Daten sendendes putmsg()");
			return 6;
		}
		printf(": ");
	}
	printf("\n");
	
	// Empfaenger-Prozess ueber Sender-Ende informieren
	cbuf[0] = ENDSEND;
	if((rc=putmsg(fd, &ctl, &data, flags)) < 0){
		perror("Beendendes putmsg()");
		return 7;
	}

	return 0;
}
